//+------------------------------------------------------------------+
//|                                        wpr + bollinger bands.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers    9
#property indicator_levelstyle STYLE_DOT
#property indicator_levelcolor clrMagenta
#property strict

//
//
//
//
//

enum enColorOn
{
   col_colorOnSlope,  // Change color on slope change
   col_colorOnLevelo, // Change color on outer bands cross
   col_colorOnLevelm  // Change color on middle band cross
};


extern ENUM_TIMEFRAMES    TimeFrame        = PERIOD_CURRENT;    // Time frame to use
extern int                CCIPeriod        = 14;                // Cci Period
extern int                BandsPeriod      = 20;                // Bands period
extern ENUM_MA_METHOD     BandsMaMode      = MODE_LWMA;         // Bands Ma Method
extern double             BandsDeviations  = 2;                 // Bands deviations   
extern double             OverSold         = -200;              // Overbought level
extern double             OverBought       = 200;               // Oversold level
extern enColorOn          ColorChangeOn    = col_colorOnLevelm; // Color change
extern color              UpBandColor      = clrLime;           // Color for upper band
extern color              MiBandColor      = clrIvory;          // Color for middle band
extern color              DnBandColor      = clrCrimson;        // Color for lower band
extern color              UpColor          = clrLimeGreen;      // Cci up color
extern color              DnColor          = clrOrangeRed;      // Cci down color
extern color              NuColor          = clrDarkSlateGray;  // Cci neutral color
extern color              ShadowColor      = clrDimGray;        // Color for "shadow"
extern int                linesWidth       = 3;                 // Lines width
extern int                shadowWidth      = 0;                 // Shadow width (0 for lines width+3)
extern bool               alertsOn         = false;             // Turn alerts on?
extern bool               alertsOnCurrent  = true;              // Alerts on current (still opened) bar?
extern bool               alertsMessage    = true;              // Alerts should display a message?
extern bool               alertsSound      = false;             // Alerts should play a sound?
extern bool               alertsEmail      = false;             // Alerts should send an email?
extern bool               alertsNotify     = false;             // Alerts should send notification?
extern bool               arrowsVisible    = false;             // Arrows visible?
extern bool               arrowsOnNewest   = false;             // Arrows drawn on newst bar of higher time frame bar?
extern string             arrowsIdentifier = "cci Arrows1";     // Unique ID for arrows
extern double             arrowsUpperGap   = 1.0;               // Upper arrow gap
extern double             arrowsLowerGap   = 1.0;               // Lower arrow gap
extern color              arrowsUpColor    = clrLimeGreen;      // Up arrow color
extern color              arrowsDnColor    = clrOrange;         // Down arrow color
extern int                arrowsUpCode     = 241;               // Up arrow code
extern int                arrowsDnCode     = 242;               // Down arrow code
extern int                arrowsSize       = 0;                 // Arrows size
extern bool               Interpolate      = true;              // Interpolate in multi time frame mode?

//
//
//
//
//

double cci[];
double cciUa[];
double cciUb[];
double cciDa[];
double cciDb[];
double bandsUp[];
double bandsMi[];
double bandsDn[];
double sdw[];
double prices[];
double value[];
string indicatorFileName;
bool   returnBars;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(11);
   int shadWidth = shadowWidth; if (shadWidth<=0) shadWidth=linesWidth+3;
   SetIndexBuffer(0, bandsUp); SetIndexStyle(0,EMPTY,STYLE_DASH,0,UpBandColor);
   SetIndexBuffer(1, bandsMi); SetIndexStyle(1,EMPTY,STYLE_DASH,0,MiBandColor);
   SetIndexBuffer(2, bandsDn); SetIndexStyle(2,EMPTY,STYLE_DASH,0,DnBandColor);
   SetIndexBuffer(3, sdw);     SetIndexStyle(3,EMPTY,EMPTY,shadWidth,ShadowColor);
   SetIndexBuffer(4, cci);     SetIndexStyle(4,EMPTY,EMPTY,linesWidth,NuColor);
   SetIndexBuffer(5, cciUa);   SetIndexStyle(5,EMPTY,EMPTY,linesWidth,UpColor);
   SetIndexBuffer(6, cciUb);   SetIndexStyle(6,EMPTY,EMPTY,linesWidth,UpColor);
   SetIndexBuffer(7, cciDa);   SetIndexStyle(7,EMPTY,EMPTY,linesWidth,DnColor);
   SetIndexBuffer(8, cciDb);   SetIndexStyle(8,EMPTY,EMPTY,linesWidth,DnColor);
   SetIndexBuffer(9, prices);
   SetIndexBuffer(10,value);
   SetLevelValue(0,OverBought);
   SetLevelValue(1,OverSold);
   

   //
   //
   //
   //
   //
   
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame==-99;
      TimeFrame         = fmax(TimeFrame,_Period);
         
   //
   //
   //
   //
   //
               
   IndicatorShortName(timeFrameToString(TimeFrame)+" Cci bands ("+(string)CCIPeriod+","+(string)BandsPeriod+","+DoubleToStr(BandsDeviations,2)+")");
   return(0);
}
int deinit()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
return(0);
}

//
//
//
//
//

int start()
{
   int i,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=fmin(Bars-counted_bars,Bars-1); 
           if (returnBars) { cci[0] = fmin(limit+1,Bars-1); return(0); }
   
   //
   //
   //
   //
   //
   
   if (TimeFrame==Period())
   {
     if (value[limit]== 1) CleanPoint(limit,cciUa,cciUb);
     if (value[limit]==-1) CleanPoint(limit,cciDa,cciDb);     
     for(i=limit; i>=0; i--) cci[i] = iCCI(NULL,0,CCIPeriod,PRICE_TYPICAL,i);
     for(i=limit; i>=0; i--)
     {
         double deviations = iStdDevOnArray(cci,0,BandsPeriod,0,BandsMaMode,i);
                bandsMi[i] = iMAOnArray(cci,0,BandsPeriod,0,BandsMaMode,i);
                bandsUp[i] = bandsMi[i]+deviations*BandsDeviations;
                bandsDn[i] = bandsMi[i]-deviations*BandsDeviations;
                cciDa[i] = EMPTY_VALUE;
                cciDb[i] = EMPTY_VALUE;
                cciUa[i] = EMPTY_VALUE;
                cciUb[i] = EMPTY_VALUE;
           if (i<Bars-1)
           {   
              value[i] = value[i+1];
              switch (ColorChangeOn)
              {
                 case col_colorOnSlope: 
                    if (cci[i]>cci[i+1])    value[i] =  1;
                    if (cci[i]<cci[i+1])    value[i] = -1;
                 break;
                 case col_colorOnLevelm: 
                    if (cci[i]>bandsMi[i])  value[i] =  1;
                    if (cci[i]<bandsMi[i])  value[i] = -1;
                 break;
                 default : 
                    value[i] = 0;
                    if (cci[i]>bandsUp[i])  value[i] =  1;
                    if (cci[i]<bandsDn[i])  value[i] = -1;
                 break;
              }    
              if (value[i]== 1) PlotPoint(i,cciUa,cciUb,cci);
              if (value[i]==-1) PlotPoint(i,cciDa,cciDb,cci);
          }
          
          //
          //
          //
          //
          //
            
          if (arrowsVisible)
          {
            string lookFor = arrowsIdentifier+":"+(string)Time[i]; ObjectDelete(lookFor);            
            if (i<(Bars-1) && value[i] != value[i+1])
            {
               if (value[i] == 1) drawArrow(i,arrowsUpColor,arrowsUpCode,false);
               if (value[i] ==-1) drawArrow(i,arrowsDnColor,arrowsDnCode, true);
            }
         }
   }   
   manageAlerts();
   return(0);
   }
   
   //
   //
   //
   //
   //
   
   limit = (int)fmax(limit,fmin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
   if (value[limit]== 1) CleanPoint(limit,cciUa,cciUb);
   if (value[limit]==-1) CleanPoint(limit,cciDa,cciDb);
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,TimeFrame,Time[i]);
         bandsUp[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,CCIPeriod,BandsPeriod,BandsMaMode,BandsDeviations,OverSold,OverBought,ColorChangeOn,UpBandColor,MiBandColor,DnBandColor,UpColor,DnColor,NuColor,ShadowColor,0,0,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotify,arrowsVisible,arrowsOnNewest,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,arrowsSize,0,y);
         bandsMi[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,CCIPeriod,BandsPeriod,BandsMaMode,BandsDeviations,OverSold,OverBought,ColorChangeOn,UpBandColor,MiBandColor,DnBandColor,UpColor,DnColor,NuColor,ShadowColor,0,0,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotify,arrowsVisible,arrowsOnNewest,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,arrowsSize,1,y);
         bandsDn[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,CCIPeriod,BandsPeriod,BandsMaMode,BandsDeviations,OverSold,OverBought,ColorChangeOn,UpBandColor,MiBandColor,DnBandColor,UpColor,DnColor,NuColor,ShadowColor,0,0,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotify,arrowsVisible,arrowsOnNewest,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,arrowsSize,2,y);
         sdw[i]     = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,CCIPeriod,BandsPeriod,BandsMaMode,BandsDeviations,OverSold,OverBought,ColorChangeOn,UpBandColor,MiBandColor,DnBandColor,UpColor,DnColor,NuColor,ShadowColor,0,0,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotify,arrowsVisible,arrowsOnNewest,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,arrowsSize,3,y);
         cci[i]     = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,CCIPeriod,BandsPeriod,BandsMaMode,BandsDeviations,OverSold,OverBought,ColorChangeOn,UpBandColor,MiBandColor,DnBandColor,UpColor,DnColor,NuColor,ShadowColor,0,0,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotify,arrowsVisible,arrowsOnNewest,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,arrowsSize,4,y);
         value[i]   = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,CCIPeriod,BandsPeriod,BandsMaMode,BandsDeviations,OverSold,OverBought,ColorChangeOn,UpBandColor,MiBandColor,DnBandColor,UpColor,DnColor,NuColor,ShadowColor,0,0,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotify,arrowsVisible,arrowsOnNewest,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,arrowsSize,10,y);
         cciDa[i]   = EMPTY_VALUE;
         cciDb[i]   = EMPTY_VALUE;
         cciUa[i]   = EMPTY_VALUE;
         cciUb[i]   = EMPTY_VALUE;
         
         //
         //
         //
         //
         //
      
         if (!Interpolate || (i>0 && y==iBarShift(NULL,TimeFrame,Time[i-1]))) continue;

         //
         //
         //
         //
         //

         int n,k; datetime time = iTime(NULL,TimeFrame,y);
         for(n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;
         for(k = 1; i+n < Bars && i+k < Bars && k < n; k++) 
         {
            bandsUp[i+k] = bandsUp[i] + (bandsUp[i+n] - bandsUp[i]) * k/n;
            bandsMi[i+k] = bandsMi[i] + (bandsMi[i+n] - bandsMi[i]) * k/n;
            bandsDn[i+k] = bandsDn[i] + (bandsDn[i+n] - bandsDn[i]) * k/n;
            sdw[i+k]     = sdw[i]     + (sdw[i+n]     - sdw[i])     * k/n;
            cci[i+k]     = cci[i]     + (cci[i+n]     - cci[i])     * k/n;      
         }               
   }
   for(i=limit; i>=0; i--)
   {
      if (value[i]== 1) PlotPoint(i,cciUa,cciUb,cci);
      if (value[i]==-1) PlotPoint(i,cciDa,cciDb,cci);
   }
return(0);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if (i>=Bars-3) return;
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
      if (first[i+2] == EMPTY_VALUE) 
            { first[i]  = from[i]; first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
      else  { second[i] = from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else     { first[i]  = from[i];                          second[i] = EMPTY_VALUE; }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      int whichBar = 1; if (alertsOnCurrent) whichBar = 0;
      if (value[whichBar] != value[whichBar+1])
      {
         switch (ColorChangeOn)
         {
            case col_colorOnSlope:
               if (value[whichBar]== 1) doAlert(whichBar,"slope changed to up");
               if (value[whichBar]==-1) doAlert(whichBar,"slope changed to down");
               break;
            case col_colorOnLevelm:
               if (value[whichBar]== 1) doAlert(whichBar,"middle band crossed up");
               if (value[whichBar]==-1) doAlert(whichBar,"middle band crossed down");
               break;
            default : 
               if (value[whichBar]== 1) doAlert(whichBar,"upper band crossed up");
               if (value[whichBar]==-1) doAlert(whichBar,"lower band crossed down");
         }    
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," ",timeFrameToString(_Period)+" Cci ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(Symbol()+" Cci ",message);
          if (alertsNotify)  SendNotification(message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+(string)Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      //
      //

      datetime time = Time[i]; if (arrowsOnNewest) time += _Period*60-1;      
      ObjectCreate(name,OBJ_ARROW,0,time,0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_WIDTH,arrowsSize);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsUpperGap * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsLowerGap * gap);
}
